/*-------------------------------------------------------------------
[SCRIPT] Change Data Capture Sample   
[DATABASE] 
[DESCRIPTION] 
[MAJ PAR] DATAFLY - Arian Papillon
[DATEMAJ] SQL Saturday - 05/09/2015  
-------------------------------------------------------------------*/
-----------------Prparer l'environnement ------------------------------
-- Base Source
USE master
GO
CREATE DATABASE CDCSource
GO
USE CDCSource
GO
SELECT  [BusinessEntityID]
      , [PersonType]
      , [FirstName]
      , [LastName]
INTO    person
FROM    AdventureWorks2014.Person.Person 
GO
ALTER TABLE person
ADD CONSTRAINT pk_person PRIMARY KEY CLUSTERED ( BusinessEntityID )
  -- Une PK ou un index unique est ncessaire pour le CDC si net changes
GO
---- Destination ---------------
CREATE DATABASE CDCDest
GO
USE CDCDest
GO
SELECT  [BusinessEntityID]
      , [PersonType]
      , [FirstName]
      , [LastName]
INTO    person
FROM    AdventureWorks2014.Person.Person 
GO
ALTER TABLE person
ADD CONSTRAINT pk_person PRIMARY KEY CLUSTERED ( BusinessEntityID )
GO

CREATE SCHEMA staging;
GO
CREATE TABLE [staging].[person_insert]
    (
      [BusinessEntityID] [INT] NOT NULL
    , [PersonType] [NCHAR](2) NOT NULL
    , [FirstName] [NVARCHAR](50) NOT NULL
    , [LastName] [NVARCHAR](50) NOT NULL
    )
ON  [PRIMARY]

CREATE TABLE [staging].[person_delete]
    (
      [BusinessEntityID] [INT] NOT NULL
    , [PersonType] [NCHAR](2) NOT NULL
    , [FirstName] [NVARCHAR](50) NOT NULL
    , [LastName] [NVARCHAR](50) NOT NULL
    )
ON  [PRIMARY]

CREATE TABLE [staging].[person_update]
    (
      [BusinessEntityID] [INT] NOT NULL
    , [PersonType] [NCHAR](2) NOT NULL
    , [FirstName] [NVARCHAR](50) NOT NULL
    , [LastName] [NVARCHAR](50) NOT NULL
    )
ON  [PRIMARY]

GO



-- Activation du Change Data Capture
USE CDCSource
GO
EXEC sys.sp_cdc_enable_db
  -- activation du CDC
GO
SELECT  name
      , is_cdc_enabled
FROM    sys.databases
WHERE   name = 'CDCSource'
GO

EXEC sys.sp_cdc_enable_table @source_schema = N'dbo', @source_name = N'person',
    @role_name = NULL					-- rle de base de donne pour limiter l'accs aux donnes de modification
-- facultatif
    , @capture_instance = 'Person_Instance1'	-- plusieurs instances de capture sur la mme table sont possibles
    , @supports_net_changes = 1			        -- interrogation des modifications nettes (consolides) : PK ncessaire, ou index unique  spcifier
    , @index_name = NULL					    -- nom d'un index pour identifier les lignes
    , @captured_column_list = NULL		        -- liste des colonnes  capturer
    , @filegroup_name = NULL				    -- filegroup, sinon par filegroup par dfaut
    , @allow_partition_switch = 1
		-- permet les switch partition si la table est partitionne
 GO
-- 
SELECT * FROM sys.objects WHERE SCHEMA_NAME(schema_id) = 'CDC' AND type_desc = 'SQL_INLINE_TABLE_VALUED_FUNCTION' AND is_ms_shipped = 0

SELECT  SCHEMA_NAME(schema_id) + '.' + name
      , *
FROM    sys.tables
WHERE   is_ms_shipped = 1 ORDER BY name



SELECT  *
FROM    [cdc].[change_tables]  -- liste des tables captures
SELECT  *
FROM    [cdc].[captured_columns]  -- liste des colonnes capturres
SELECT  *
FROM    [cdc].[index_columns] -- cls
SELECT  *
FROM    [cdc].[ddl_history]  -- historique DDL
SELECT  *
FROM    [cdc].[lsn_time_mapping] -- lsn / time mapping (mme sans activit ttes les 5 mn)
SELECT  *
FROM    [dbo].[systranschemas] -- suivi des modifications de schma

SELECT  *
FROM    [cdc].[Person_Instance1_CT]

-- Si j'ajoute une colonne  la table, elle n'est pas prise en compte
-- ALTER TABLE dbo.person ADD Comment VARCHAR(MAX) NULL

-- comment rcuprer les lsn
SELECT  sys.fn_cdc_get_min_lsn('Person_Instance1') AS min_lsn
SELECT  sys.fn_cdc_get_max_lsn()
SELECT  sys.fn_cdc_map_time_to_lsn('largest less than or equal', GETDATE())


--- Effectuons des mises  jour --------------------
-- Modifier
UPDATE  person
SET     FirstName = 'Steve'
      , LastName = 'Ballmer'
WHERE   BusinessEntityID = 1
GO 

UPDATE  person
SET     FirstName = 'Satya'
      , LastName = 'Nadella'
WHERE   BusinessEntityID = 1
GO

INSERT  person
        ( BusinessEntityID
        , PersonType
        , FirstName
        , LastName
        )
VALUES  ( 310
        , 'SC'
        , 'Joan'
        , 'Ballmer'
        )
GO

UPDATE  person
SET     FirstName = 'Bill', LastName='Gates'
WHERE   BusinessEntityID = 310
GO
DELETE  person
WHERE   BusinessEntityID = 2
GO

-- Rcuprer les modifications
DECLARE @from_lsn BINARY(10)
  , @to_lsn BINARY(10)
SET @from_lsn = sys.fn_cdc_get_min_lsn('Person_Instance1')
SET @to_lsn = sys.fn_cdc_get_max_lsn()
SELECT  *
      , CHOOSE(__$operation, 'DELETE', 'INSERT', 'UPDATE (before)',
               'UPDATE (after)')
FROM    cdc.fn_cdc_get_all_changes_Person_Instance1(@from_lsn, @to_lsn,
                                                    'All update old')
SELECT  *
      , CHOOSE(__$operation, 'DELETE', 'INSERT', 'UPDATE (before)',
               'UPDATE (after)')
FROM    cdc.fn_cdc_get_all_changes_Person_Instance1(@from_lsn, @to_lsn, 'All')
SELECT  *
      , CHOOSE(__$operation, 'DELETE', 'INSERT', 'UPDATE (before)',
               'UPDATE (after)')
FROM    cdc.fn_cdc_get_net_changes_Person_Instance1(@from_lsn, @to_lsn, 'All')


-- dsactiver
EXEC sys.sp_cdc_disable_table @source_schema = N'dbo',
    @source_name = N'Person', @capture_instance = N'Person_Instance1'
GO
EXEC sys.sp_cdc_disable_db